/*		 ITI-FG  Kernel module free public release
 *
 *		     Copyright (C) 1996-2000 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: itifgBuf.c,v 7.0.2.1 2000/11/29 15:00:36 cvs Exp $
 *
 */

/*
 * Includes:
 */

#include "itifgOS.h"	/* os specific stuff */
#include "itifgExt.h"	/* offical driver interface */
#include "itifgReg.h"	/* register definitions ICR Regs */
#include "icpciReg.h"	/* register definitions IC-PCI */
#include "amvsReg.h"	/* register definitions AM-VS */
#include "amdigReg.h"	/* register definitions AM-DIG */
#include "ampcvReg.h"	/* register definitions PCVision */
#include "pcdigReg.h"	/* register definitions PC-DIG */
#include "itifgIntD.h"	/* driver internals */
#include "itifgIntF.h"	/* internal functions */

/*
 * Defines:
 */

/*
 * Globals:
 */

/*
 * iti_copy
 */

int
iti_bma_restart (itifg_t *iti)
{
  int iov;

  pcd_dmadata_t* pcd_dma_table = (pcd_dmadata_t*)iti->pcd_dma_table;

  if (!iti->started)
    return ERROR;

  if (IS_PCDIG)
    if (iti->enabled)
      return ERROR;

  ITI_PRINT5A ("  C%d B%d V%d\n", ACNT(cam), ACNT(blk), ACNT(iov));

  if (IS_ICPCI)
    {
      if (ACNT(cam) >= DCNT(cam) || ACNT(blk) >= DCNT(blk) ||
	  ACNT(iov) >= DCNT(iov))
	iti_printwa ("a) try to transfer invalid block at: C%d/B%d/V%d\n",
		     ACNT(cam), ACNT(blk), ACNT(iov));
      else
	if (iti->sglist[ACNT(cam)][ACNT_OFFSET(blk) + ACNT(iov)]
	    == NULL)
	  iti_printwa ("b) try to transfer invalid block at: C%d/B%d/V%d\n",
		       ACNT(cam), ACNT(blk), ACNT(iov));
	else
	  {
	    icr_set_bmdst (iti, (u_long)iti->sglist[ACNT(cam)]
						   [ACNT_OFFSET(blk) +
						   ACNT(iov)]);
	    icr_set_bmxc (iti, PAGE_SIZE);
	  }
    }
  else
    if (IS_PCDIG)
      {
	CHECK_FOR_EIO (pcd_set_dmaacc, PCD_ENABLE);
	for (iov = 0; iov < DCNT(iov); iov++)
	  {
	    if (ACNT(cam) >= DCNT(cam) || ACNT(blk) >= DCNT(blk) ||
		ACNT(iov) >= DCNT(iov))
	      iti_printwa ("a) try to transfer invalid block at:"
			   " C%d/B%d/V%d\n", ACNT(cam), ACNT(blk), iov);
	    else
	      if (iti->sglist[ACNT(cam)][ACNT_OFFSET(blk) + iov]
		  == NULL)
		iti_printwa ("b) try to transfer invalid block at:"
			     " C%d/B%d/V%d\n", ACNT(cam), ACNT(blk), iov);
	      else
		{
		  iti_writel ((u_long)iti->sglist[ACNT(cam)]
						 [ACNT_OFFSET(blk) + iov],
			      &pcd_dma_table[iov].addr);
		  iti_writel (PAGE_SIZE, &pcd_dma_table[iov].xcnt);
		}
	  }
	CHECK_FOR_EIO (pcd_set_dmaacc, PCD_DISABLE);
	CHECK_FOR_EIO (pcd_set_octstart, ACNT(blk) *
		       (ITI_BLOCK_SIZE / PCD_OCT_SPLIT_SIZE));
	pcd_set_segsize (iti, DCNT(iov));
      }

  return OK;
}

int
iti_bma_start (itifg_t *iti)
{
  ITI_PRINT5("  bma strt\n");
  
  if (IS_ICPCI)
    {
      CHECK_FOR_EIO (cr_set_bmsf, iti->bma1stfld);
      CHECK_FOR_EIO (cr_set_bmss, iti->bma2ndfld);
      
      iti_bma_restart (iti);
      
      icr_clear_fifoclr (iti);
      cr_set_bmstren (iti, CR_ENABLE);
      icr_set_bmreq (iti, ICR_ENABLE);
    }
  else
    if (IS_PCDIG)
      {
	if (iti_bma_restart (iti))
	  return OK;

	pcd_set_bmenb (iti, PCD_ENABLE);
	iti->enabled = TRUE;
      }

  return OK;
}

int
iti_bma_stop (itifg_t *iti)
{
  ITI_PRINT5("  bma stop\n");
  
  if (IS_ICPCI)
    {
      cr_set_bmstren (iti, CR_DISABLE);
      icr_set_bmreq (iti, ICR_DISABLE);
    }
  else
    if (IS_PCDIG)
      {
	pcd_set_bmenb (iti, PCD_DISABLE);
	iti->enabled = FALSE;
      }
  
  return OK;
}

/*
 * iti_clear
 */

void
iti_clear (itifg_t *iti,
	   caddr_t base,
	   size_t size,
	   iti_clear_t mode,
	   int user_space)
{
  int i, j;

  caddr_t temp = NULL;
  u_short value;

  ITI_PRINT1A("> iti_clear: at 0x%p,\n", base);
  ITI_PRINT1A("    " SIZE_F " bytes - user %s\n", size,
	      user_space ? "TRUE" : "FALSE");

  if (user_space)
    {
      temp = iti_malloc (PAGE_SIZE, ITI_UNPAGED);
      ITI_PRINT3A("    temp clear: 0x%p\n", temp);
    }
  if (mode == ITI_CLEAR_BLACK)
    {
      if (user_space)
	goto done;
      else
	iti_memset (base, 0x00, size);
    }
  else
    if (mode == ITI_CLEAR_GREY)
      switch (iti->get_pixsz (iti))
	{
	case 1:
	case 3:
	  if (user_space)
	    {
	      iti_memset (temp, 0x80, PAGE_SIZE);
	      for (j = 0; j < size / PAGE_SIZE; j++, base += PAGE_SIZE)
		iti_copyout (base, temp, PAGE_SIZE);
	      if (size % PAGE_SIZE)
		iti_copyout (base, temp, size % PAGE_SIZE);
	    }
	  else
	    iti_memset (base, 0x80, size);
	  break;
	case 2:
	  switch (MODULE_ID)
	    {
	    case ICP_AMDG:
	    case ITI_VMPCD:
	      switch (pcd_get_lut0psz (iti))
		{
		case 16:
		  value = 0x8000;
		  break;
		case 14:
		  value = 0x2000;
		  break;
		case 12:
		  value = 0x0800;
		  break;
		case 10:
		  value = 0x0200;
		  break;
		default:
		  goto done;
		}
	      break;
	    default:
	      goto done;
	    }
	  if (user_space)
	    {
	      for (i = 0; i < PAGE_SIZE / sizeof (u_short); i++)
		((u_short*)temp)[i] = value;
	      for (j = 0; j < size / PAGE_SIZE; j++, base += PAGE_SIZE)
		iti_copyout (base, temp, PAGE_SIZE);
	      if (size % PAGE_SIZE)
		iti_copyout (base, temp, size % PAGE_SIZE);
	    }
	  else
	    for (i = 0; i < size / sizeof (u_short); i++)
	      ((u_short*)base)[i] = value;
	  break;
	case 4:
	  if (user_space)
	    {
	    for (i = 0; i < PAGE_SIZE / sizeof (u_long); i++)
	      ((u_long*)temp)[i] = 0x80000000;
	      for (j = 0; j < size / PAGE_SIZE; j++, base += PAGE_SIZE)
		iti_copyout (base, temp, PAGE_SIZE);
	      if (size % PAGE_SIZE)
		iti_copyout (base, temp, size % PAGE_SIZE);
	    }
	  else
	    for (i = 0; i < size / sizeof (u_long); i++)
	      ((u_long*)base)[i] = 0x80000000;
	  break;
	}
done:
  
  if (user_space)
    {
      ITI_PRINT3A("    temp clear: 0x%p\n", temp);
      iti_free (temp, ITI_UNPAGED);
    }

  ITI_PRINT1("< iti_clear\n");
}

/*
 * iti_clear_sglist
 */

int
iti_clear_sglist (itifg_t *iti)
{
  int snap;

  if (!iti) return ERROR;
  
  for (snap = 0; snap < ITI_SNAPS_MAX; snap++)
    iti_memset (iti->sglist[snap], 0, ITI_SGLIST_SIZE);
  return OK;
}

/*
 * iti_src_init
 */

int
iti_src_init (itifg_t *iti)
{
  int piece;

  u_long buf_base;
  size_t raw_size;

  pcd_octdata_t* pcd_oct_table = (pcd_octdata_t*)iti->pcd_oct_table;

  pcd_octdata_low_t low_tmp = { 0UL };
  pcd_octdata_high_t high_tmp = { 0UL };

  ITI_PRINT1("> iti_src_init\n");

  buf_base = 0;
  ITI_PRINT1A("    buf_base    %#lx\n", buf_base);
  raw_size = iti->raw_size;
  ITI_PRINT1A("    raw_size    " SIZE_F "\n", raw_size);
  
  switch (DEVICE_ID)
    {
    case ITI_ICPCI:
      /* disable pending transfers */
      CHECK_FOR_EIO (cr_set_bmstren, CR_DISABLE);
      CHECK_FOR_EIO (icr_set_bmreq, ICR_DISABLE);
      /* set source values */
      CHECK_FOR_EIO (cr_set_pitch, (iti->width *
				    cr_get_pixsz(iti)));
      CHECK_FOR_EIO (cr_set_aoix, (iti->width *
				   cr_get_pixsz(iti)) - 8);
      if (iti->get_smstat (iti) == CR_ILACE &&
	  iti->get_vdec (iti) == CR_DEC_BY_1)
	{
	  CHECK_FOR_EIO (cr_set_bmilace, CR_ENABLE);
	  iti->bma1stfld = buf_base;
	  iti->bma2ndfld = buf_base +
	    iti->raw_size / 2;
	}
      else
	{
	  CHECK_FOR_EIO (cr_set_bmilace, CR_DISABLE);
	  iti->bma1stfld = buf_base;
	  iti->bma2ndfld = buf_base;
	}
      break;
      
    case ITI_PCDIG:
      /* disable pending transfers */
      CHECK_FOR_EIO (pcd_set_bmenb, PCD_DISABLE);
      /* fill in the OC table */
      CHECK_FOR_EIO (pcd_set_octacc, PCD_ENABLE);
      ITI_PRINT1A("    split_size  %#x\n", PCD_OCT_SPLIT_SIZE);

      /* FIXME */
      switch ((enum pcd_geo_t)iti->dev.pcd_cam.geometry)
	{
	case PCD_1CHAN_L2R:
	case PCD_2CHAN_INTERLEAVED:
	case PCD_2CHAN_SEP_TAP_L2R:
	case PCD_2CHAN_SEP_TAP_R2L:
	case PCD_2CHAN_SEP_TAP_CONVERGE:
	case PCD_2TAP_INTERLINE_A_EVEN:
	case PCD_2TAP_INTERLINE_B_EVEN:
	case PCD_SEP_QUAD_TAP_CONVERGE:
	case PCD_4CHAN_SEP_TAP_L2R:
	case PCD_4CHAN_SEP_TAP_R2L:
	case PCD_4CHAN_2SEG_INTERLEAVED:
	case PCD_4CHAN_INTERLEAVE_CONVERGE:
	  break;
	}
      
      piece = 0;
      raw_size = iti->raw_size;
      high_tmp.b.swap = FALSE;
      high_tmp.b.decr = FALSE;
      high_tmp.b.last = FALSE;
      high_tmp.b.xcnt = (PCD_OCT_SPLIT_SIZE >> PCD_OCT_SPLIT_SHIFT) - 1;
      while (TRUE)
	{
	  if (raw_size < PCD_OCT_SPLIT_SIZE)
	    break;
	  low_tmp.b.addr = (buf_base >> PCD_OCT_LOW_ADDR_SHIFT) &
	    PCD_OCT_LOW_ADDR_MASK;
	  iti_writel (low_tmp.all, &pcd_oct_table[piece].low.all);
	  high_tmp.b.addr = buf_base >> PCD_OCT_HIGH_ADDR_SHIFT;
	  iti_writel (high_tmp.all, &pcd_oct_table[piece].high.all);
	  buf_base += PCD_OCT_SPLIT_SIZE;
	  raw_size -= PCD_OCT_SPLIT_SIZE;
	  piece++;
	}
      if (raw_size)
	{
	  low_tmp.b.addr = buf_base & PCD_OCT_LOW_ADDR_MASK;
	  iti_writel (low_tmp.all, &pcd_oct_table[piece].low.all);
	  high_tmp.b.addr = buf_base >> PCD_OCT_HIGH_ADDR_SHIFT;
	  high_tmp.b.last = TRUE;
	  high_tmp.b.xcnt = (raw_size >> PCD_OCT_SPLIT_SHIFT) - 1;
	  iti_writel (high_tmp.all, &pcd_oct_table[piece].high.all);
	  buf_base += raw_size;
	  piece++;
	}
      else
	{
	  high_tmp.all =
	    iti_readl (&pcd_oct_table[piece - 1].high.all);
	  high_tmp.b.last = TRUE;
	  iti_writel (high_tmp.all,
		      &pcd_oct_table[piece - 1].high.all);
	}
      ITI_PRINT1A("    num_entries %d\n", piece);
      CHECK_FOR_EIO (pcd_set_octacc, PCD_DISABLE);
      CHECK_FOR_EIO (pcd_set_octstart, 0);
      CHECK_FOR_EIO (pcd_set_bmstep, PCD_BMSTEP_1DDWORD);
      break;
    }

  ITI_PRINT1("< iti_src_init\n");  
  return OK;
}

/*
 * iti_dst_init
 */

int
iti_dst_init (itifg_t *iti, caddr_t arg_base, size_t arg_size, int user_space)
{
  int error, frame, frames, block, page;

  caddr_t base = arg_base;
  u_long vect;
  size_t size = arg_size;

  size_t frame_raw, frame_paged, frame_off;
  size_t block_size, frst_block_size = 0, last_block_size;
  int blocks;
  size_t page_size, frst_page_size = 0, full_page_size, last_page_size = 0;
  int pages, frst_pages = 0, full_pages, last_pages = 0;
  
  int start_offset;

  ITI_PRINT1("> iti_dst_init\n");
  
  iti_clear_sglist (iti);

  frame_raw = iti->raw_size;
  frame_paged = iti->paged_size;
  frame_off = frame_paged - frame_raw;
  frames = arg_size / frame_paged;
  if (frames > ITI_SNAPS_MAX)
    return -ENOMEM;
  
  block_size = ITI_BLOCK_SIZE;
  if (iti->get_smstat (iti) == CR_ILACE &&
      iti->get_vdec (iti) == CR_DEC_BY_1)
    block_size *= 2;
  full_page_size = PAGE_SIZE;
  full_pages = block_size / full_page_size;
  blocks = 0;

  if (frame_raw > block_size)
    {
      if (iti->get_smstat (iti) == CR_ILACE &&
	  iti->get_vdec (iti) == CR_DEC_BY_1)
	{
	  frst_block_size = (ITI_BLOCK_SIZE -
			     ((frame_raw / 2) % ITI_BLOCK_SIZE));
	  frst_block_size *= 2;
	  if (!frst_block_size)
	    frst_block_size = block_size;
	}
      else
	frst_block_size = block_size;
      blocks++;
    }
      
  if (frst_block_size)
    {
      frst_pages = frst_block_size / full_page_size;
      frst_page_size = frst_block_size % full_page_size;
    }

  last_block_size = frame_raw % block_size;

  if (iti->get_smstat (iti) == CR_ILACE &&
      iti->get_vdec (iti) == CR_DEC_BY_1)
    {
      last_block_size *= 2;
      last_block_size += frst_page_size;
      frst_block_size -= frst_page_size;
    }
  if (!last_block_size)
    last_block_size = block_size;
  blocks++;
  
  blocks += (frame_raw - frst_block_size - last_block_size) / block_size;
  
  if (!frst_pages && blocks != 1)
    frst_pages = full_pages;

  if (last_block_size < block_size * 3 / 4 && blocks > 1)
    {
      if (blocks == 2)
	{
	  last_block_size += frst_block_size;
	  frst_block_size = 0;
	  frst_pages = 0;
	}
      else
	last_block_size += block_size;	
      blocks--;
    }
  
  if (last_block_size)
    {
      last_pages = last_block_size / full_page_size;
      last_page_size = last_block_size % full_page_size;
      if (last_page_size)
	last_pages++;
      else
	last_page_size = full_page_size;
    }
  
  if (blocks == 1)
    start_offset = iti->ob_size / ITI_BLOCK_SIZE - 1;
  else
    if (iti->get_smstat (iti) == CR_ILACE &&
	iti->get_vdec (iti) == CR_DEC_BY_1)
      {
	start_offset = (frame_raw / 2) / ITI_BLOCK_SIZE;
	if ((frame_raw / 2) % ITI_BLOCK_SIZE)
	  start_offset++;
      }
    else
      start_offset = 1;
  
  ITI_PRINT1A("    buffer size " SIZE_F ".\n", size);
  ITI_PRINT1A("    -> %d frame(s).\n", frames);
  ITI_PRINT1A("    frame size " SIZE_F " - " SIZE_F "\n", frame_paged,
	      frame_raw);
  ITI_PRINT1A("      = " SIZE_F ".\n", frame_off);

  ITI_PRINT1A("    block size " SIZE_F ".\n", block_size);
  ITI_PRINT1A("    page size " SIZE_F ".\n", full_page_size);
  ITI_PRINT1A("    -> %d page(s).\n", full_pages);

  ITI_PRINT1A("    start offset %d.\n", start_offset);
  ITI_PRINT1A("    first block size " SIZE_F ".\n", frst_block_size);
  ITI_PRINT1A("    -> %d page(s).\n", frst_pages);

  ITI_PRINT1A("    last block size " SIZE_F ".\n", last_block_size);
  ITI_PRINT1A("    -> %d page(s).\n", last_pages);
  ITI_PRINT1A("    last page size " SIZE_F ".\n", last_page_size);

  ITI_PRINT1A("    -> %d block(s).\n", blocks);

  iti->cam_desired = frames;  

  iti->full = full_pages;
  iti->frst = frst_pages;
  iti->last = last_pages;
  iti->iov_desired = 0;

  iti->offset = start_offset;
  iti->blk_desired = blocks;  
  
  for (frame = 0; frame < frames; frame++)
    {
      for (block = 0; block < blocks; block++)
	{
	  if (block == blocks - 1)
	    pages = last_pages;
	  else
	    if (!block)
	      pages = frst_pages;
	    else
	      pages = full_pages;
	  
	  for (page = 0; page < pages; page++)
	    {
	      if ((error =  iti_virt_to_phys (base, NULL, &vect, user_space)))
		return error;
	      ITI_PRINT3A("  C%d B%d V%d\n", frame, block, page);
	      ITI_PRINT3A("    v:0x%p->p:%#lx\n", base, vect);
	      iti->sglist[frame][block * iti_blk_size + page] = (caddr_t)vect;
	      page_size = full_page_size;
	      if (block == blocks - 1 && page == pages - 1)
		page_size = last_page_size;
	      base += page_size;
	      size -= page_size;
	    }
	}
      base += frame_off;
      size -= frame_off;
    }

  /* something goes wrong during calculation */
  if (size)
    {
      ITI_PRINT1A("    remaining size: " SIZE_F "\n", size);
      return -EINVAL;
    }

  ITI_PRINT1("< iti_dst_init\n");
  return OK;
}

/*
 * iti_fmt_init
 */

int
iti_fmt_init (itifg_t *iti)
{
  ITI_PRINT1("> iti_fmt_init\n");
  
  /* switch to module specific subroutines for width / height / interlaced */
  switch (MODULE_ID)
    {
    case ICP_AMVS:
      iti->width = vs_get_hsz (iti);
      iti->height = vs_get_vsz (iti);
      if (vs_get_smode (iti) == VS_ILACE)
	iti->height <<= 1;
      break;
    case ICP_AMDG:
      iti->width = dg_get_hact (iti);
      if (dg_get_ilpixel (iti) == DG_ILEAVE)
      iti->width <<= 1;
      iti->height = dg_get_vact (iti);
      if (dg_get_ilmode (iti) == DG_ILACE)
	iti->height <<= 1;
      break;
    case ICP_AMPV:
      iti->width =
	pv_get_hactive (iti) - pv_get_hoffset (iti) + 2;
      iti->height =
	pv_get_vactive (iti) - pv_get_voffset (iti);
      if (pv_get_smode (iti) == PV_ILACE)
	iti->height <<= 1;
      break;
    case ITI_VMPCD:
      if (FPGA_OK)
	{
	  iti->width = pcd_get_hact (iti);
	  iti->height = pcd_get_vact (iti);
	  switch (pcd_get_sform (iti))
	    {
	    case PCD_SFORM_8BIT_ST:
	    case PCD_SFORM_16BIT_ST:
	    case PCD_SFORM_24BIT_PST:
	    case PCD_SFORM_32BIT_ST:
	      switch ((enum pcd_geo_t)iti->dev.pcd_cam.geometry)
		{
		case PCD_1CHAN_L2R:
		  break;
		case PCD_2CHAN_INTERLEAVED:
		case PCD_2CHAN_SEP_TAP_L2R:
		case PCD_2CHAN_SEP_TAP_R2L:
		case PCD_2CHAN_SEP_TAP_CONVERGE:
		case PCD_2TAP_INTERLINE_A_EVEN:
		case PCD_2TAP_INTERLINE_B_EVEN:
		case PCD_SEP_QUAD_TAP_CONVERGE:
		case PCD_4CHAN_SEP_TAP_L2R:
		case PCD_4CHAN_SEP_TAP_R2L:
		case PCD_4CHAN_2SEG_INTERLEAVED:
		case PCD_4CHAN_INTERLEAVE_CONVERGE:
		  return -EINVAL;
		}
	      break;
	    case PCD_SFORM_8BIT_DT:
	    case PCD_SFORM_16BIT_DT:
	      switch ((enum pcd_geo_t)iti->dev.pcd_cam.geometry)
		{
		case PCD_1CHAN_L2R:
		  break; /* default */
		case PCD_2CHAN_INTERLEAVED:
		case PCD_2CHAN_SEP_TAP_L2R:
		case PCD_2CHAN_SEP_TAP_R2L:
		case PCD_2CHAN_SEP_TAP_CONVERGE:
		  iti->width *= 2;
		  break;
		case PCD_2TAP_INTERLINE_A_EVEN:
		case PCD_2TAP_INTERLINE_B_EVEN:
		  iti->height *= 2;
		  break;
		case PCD_SEP_QUAD_TAP_CONVERGE:
		case PCD_4CHAN_SEP_TAP_L2R:
		case PCD_4CHAN_SEP_TAP_R2L:
		case PCD_4CHAN_2SEG_INTERLEAVED:
		case PCD_4CHAN_INTERLEAVE_CONVERGE:
		  return -EINVAL;
		}
	      break;
	    case PCD_SFORM_8BIT_QT:
	      switch ((enum pcd_geo_t)iti->dev.pcd_cam.geometry)
		{
		case PCD_1CHAN_L2R:
		  break; /* default */
		case PCD_2CHAN_INTERLEAVED:
		case PCD_2CHAN_SEP_TAP_L2R:
		case PCD_2CHAN_SEP_TAP_R2L:
		case PCD_2CHAN_SEP_TAP_CONVERGE:
		case PCD_2TAP_INTERLINE_A_EVEN:
		case PCD_2TAP_INTERLINE_B_EVEN:
		  return -EINVAL;
		case PCD_SEP_QUAD_TAP_CONVERGE:
		  iti->width *= 2;
		  iti->height *= 2;
		case PCD_4CHAN_SEP_TAP_L2R:
		case PCD_4CHAN_SEP_TAP_R2L:
		case PCD_4CHAN_2SEG_INTERLEAVED:
		case PCD_4CHAN_INTERLEAVE_CONVERGE:
		  iti->width *= 4;
		  break;
		}
	      break;
	    }
	}
      else
	{
	  /* fall back to init values */
	  iti->width = PCD_HACT_INIT;
	  iti->height = PCD_VACT_INIT;
	}
      break;
    default:
      return -ENXIO;
    }

  if (FPGA_OK) iti->width >>= iti->get_hdec (iti);
    /* else assume no decimation */
  if (FPGA_OK) iti->height >>= iti->get_vdec (iti);
    /* else assume no decimation */
  iti->raw_size = iti->width * iti->height;
  if (FPGA_OK) iti->raw_size *= iti->get_pixsz (iti);
    /* else assume 8bpp */
  
  ITI_PRINT1A("    width: %d, height: %d\n", iti->width, iti->height);
  ITI_PRINT1A("    bpp: %d, size: " SIZE_F "\n",
	      (int)(iti->raw_size / iti->width / iti->height * 8),
	      iti->raw_size); 
  
  /* align buffers to 4k (minimal hardware limit) */
  if (PAGE_SIZE < (1 << CR_AQSTART_SHIFT))
    {
      iti_printw ("The system PAGE_SIZE is too small.\n");
      return -EINVAL;
    }
  iti->paged_size =
    ROUND_UPTO_PAGE(iti->raw_size + sizeof(struct iti_info_t));
  if (iti->paged_size > ITI_SIZE_MAX)
    {
      iti_printn ("Maximum frame size exeeded.\n");
      iti_printn ("Please increase ITI_SIZE_MAX.\n");
      return -ENOMEM;
    }

  ITI_PRINT1("< iti_fmt_init\n");
  return OK;
}  

/*
 * iti_buf_init
 */

int
iti_buf_init (itifg_t *iti)
{
  int snap;

  ITI_PRINT1("> iti_buf_init\n");
  
  /* fillin structure */
  iti->ob_base = iti->v_mem_base;
  switch (DEVICE_ID)
    {
    case ITI_ICPCI:
      if (iti->big_mem)
	iti->ob_size = ICP_MEM_MAP_SIZE << 1;
      else
	iti->ob_size = ICP_MEM_MAP_SIZE;
      break;
    case ITI_PCDIG:
      iti->ob_size = PCD_MEM_MAP_SIZE;
      break;
    default:
      return -ENXIO;
    }

  iti->km_size = 0;
  iti->km_base = NULL;

  ITI_PRINT3A("    sglist: %ld\n", ITI_SGLIST_SIZE);
  for (snap = 0; snap < ITI_SNAPS_MAX; snap++)
    {
      iti->sglist[snap] = (caddr_t*)iti_malloc (ITI_SGLIST_SIZE, ITI_UNPAGED);
      if (iti->sglist[snap] == NULL)
	return -ENOMEM;
      ITI_PRINT3A("    sglist[%d]: 0x%p\n", snap, iti->sglist[snap]);
    }
  iti_clear_sglist (iti);
  
  ITI_PRINT1("< iti_buf_init\n");
  return OK;
}

