/*		 ITI-FG  Kernel module free public release
 *
 *		     Copyright (C) 1996-2000 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 *
 * $Id: itifgIntD.h,v 7.0 2000/10/25 09:07:57 cvs Exp $
 *
 */

#ifndef _ITIFGINTD_H
#define _ITIFGINTD_H

#if defined(linux) && defined(__KERNEL__) || \
    defined(__NetBSD__) && defined(_KERNEL) || \
    !defined(OS_DEFINED)
/*
 * Includes:
 */

/*
 * Defines
 */

/* minor device number encoding */
#define ITI_BOARD_MASK	0xF0	/* up to 16 boards supported */
#define ITI_BOARD_SHIFT	4
#define ITI_FUNCT_MASK	0x0F	/* every with 16 different functionalities */
#define ITI_FUNCT_SHIFT	0

/* available functionalities */
#define ITI_FUNCT_DATA	0	/* the normal grabber device */
#define ITI_FUNCT_PORT	1	/* the on board io-ports */

/* some driver starup defines */
#define ICP_MEM_MAP_SIZE	0x00200000 /* Frame Grabber on Board Memory
					      Size 2 Mb default */
#define PCD_MEM_MAP_SIZE	0x00400000 /* Frame Grabber on Board Memory
					      Size 4 Mb default */
#define ICP_MOD_MAP_SIZE	(2 * PAGE_SIZE) /* 8k Acquisition Module
						   Register Space (2 Pages)*/
     
#define ICP_MODID_OFFST	0x00	/* where to find the modid in reg space */
#define ICP_TYPE_MASK	0x00F8	/* decoded modid bits */
#define ICP_TYPE_SHIFT	3	/* shifts for a useful value */

#define ICP_MEMSZ_MASK	0x80	/* on board memory size is decoded */
#define ICP_MEMSZ_SHIFT	7	/* in revision id register */
#define ICP_LEVEL_MASK	0x80	/* revision 1 level is decoded */
#define ICP_LEVEL_SHIFT	7	/* in status register */

/* interrupts */
/* additional PCVision */
#define PV_SOTINT	0x00000001
#define PV_EOTINT	0x00000002
#define PV_EIN0INT	0x00000004
#define PV_EIN1INT	0x00000008
/* additional PCDig */
#define PCD_ADRINT	0x00000010
#define PCD_BMINT	0x00000020
#define PCD_SWCINT	0x00000040
/* base IC-PCI */
#define CR_VBINT	0x00000080
#define CR_OVBINT	0x00000100
#define CR_EVBINT	0x00000200
#define CR_AMINT	0x00000400
#define CR_TRIGINT	0x00000800
#define CR_ADRINT	0x00001000
#define CR_AQINT	0x00002000
/* emulated IC-PCI */
#define CR_VB_FLD	0x00004000
/* newer IC-PCI */
#define CR_INPINT	0x00008000
/* base IC-PCI and PCDig */
#define ICR_MINT	0x00020000
#define ICR_BINT	0x00040000
#define ICR_MAINT	0x00100000
#define ICR_TAINT	0x00200000
/* base PCDig */
#define PCD_VBINT	0x01000000
#define PCD_HBINT	0x02000000
#define PCD_TRIG0INT	0x04000000
#define PCD_TRIG1INT	0x08000000
#define PCD_SOAQINT	0x10000000
#define PCD_EOAQINT	0x20000000
#define PCD_PIOINT	0x40000000

/* operation modes and flags */
#define ITI_MODE_NONE  		000	/* no operation pending */
#define ITI_MODE_GRAB		001	/* acqusition into kernel buffers */
#define ITI_MODE_SNAP		002	/* acqusition into user buffers */
#define ITI_MODE_MASK		003	/* operation is an image acqusition */
#define ITI_WAIT_SYNC		004	/* wait for the next vsync/timer irq */
#define ITI_WAIT_MASK		007	/* operation uses sleep/wakup/timer */
#define ITI_SNAP_DONE		010	/* read has done */
#define ITI_TIMEOUT		020	/* no vsync in expire period */

/*
 * Shortcuts:
 */

#define DEVICE_ID	iti->device_id
#define MODULE_ID	iti->module

#define FPGA_OK		iti->fpga_ok

#define IS_DATA		(funct == ITI_FUNCT_DATA)
#define IS_PORT		(funct == ITI_FUNCT_PORT)

#define ITI_ICPCI	ITI_PCI_DEVICE_ID_IMAGINGTECH_ICPCI
#define IS_ICPCI	(iti->device_id == ITI_ICPCI)
#define IS_REV_A	(IS_ICPCI && iti->revision < ICP_REV_B)
#define IS_REV_B	(IS_ICPCI && iti->revision > ICP_REV_A)

#define IS_AMVS		(iti->module == ICP_AMVS)
#define IS_AMDG		(iti->module == ICP_AMDG)
#define IS_AMPV		(iti->module == ICP_AMPV)

#define IS_PV_REV_A	(IS_AMPV && iti->revision < PV_REV_B)
#define IS_PV_REV_B	(IS_AMPV && iti->revision > PV_REV_A)

#define ITI_PCDIG	ITI_PCI_DEVICE_ID_IMAGINGTECH_PCDIG
#define IS_PCDIG	(iti->device_id == ITI_PCDIG)
#define IS_VMPCD	(iti->module == ITI_VMPCD)

#define iti_cam_size	(ITI_SIZE_MAX / ITI_BLOCK_SIZE * 2)
#define iti_blk_size	(ITI_BLOCK_SIZE / PAGE_SIZE * 2)
  
#define ITI_SGLIST_SIZE	(iti_cam_size * iti_blk_size)
  
/* actual and desired counts of iovec's */
#define ACNT(str)	iti-> ## str ## _actual
#define DCNT(str)	iti-> ## str ## _desired
#define ACNT_OFFSET(str)	ACNT(str) * iti_ ## str ## _size
#define ZERO_ACNT(str)	ACNT(str) = 0
#define ZERO_DCNT(str)	DCNT(str) = 0
#define IS_ACNT_ZERO(str)	(ACNT(str) == 0)
#define IS_DCNT_ZERO(str)	(DCNT(str) == 0)
#define INCR_ACNT(str)	ACNT(str)++
#define INCR_DCNT(str)	DCNT(str)++
#define SET_REACHED(str)	ACNT(str) = DCNT(str)
#define HAS_REACHED(str)	(ACNT(str) >= DCNT(str))

#define CHECK_FOR_EIO(function, argument) \
	if (function (iti, argument) < 0) \
	  { \
	    iti_error (iti, # function); \
	    return -EIO; \
	  }

/*
 * Typedefs:
 */

/* global overall structure */
struct _itifg_t
{
  int errno;		/* global error number */
  int used_by_me;	/* board is under driver control */
  
  /* operating system private */
  struct iti_osp_t osp;
  /* camera configuration */
  union	 iti_dev_t dev;
  /* accounting */
  struct iti_acc_t acc;
  /* scatter-gather list */
  caddr_t *sglist[ITI_SNAPS_MAX];
     
  /* global driver structures */
  /* own copy of per board pci config space */
  u_short device_id;
  u_short status;
  u_char revision;
  u_long address[6];
  u_char irq_line;

  u_long *v_icr_base;	/* remapped address0 */
  u_char *v_cr_base;	/* remapped address1 (can reference u_short/u_long) */
  u_short *v_mod_base;	/* remapped address2 */
  int big_mem;		/* flag: 4Mb on board memory (default 2Mb) */
  u_char *v_mem_base;	/* remapped address3 */
  int regs_io;		/* flag: icr/cr regs are not mmaped */
  u_short io_icr_base;	/* address0 is i/o space (replaces v_icr_base) */
  u_short io_cr_base;	/* address1 is i/o space (replaces v_cr_base) */
  struct pv_reg_t *pv_reg; /* pointer to PCVision registers */
  struct pcd_bi_reg_t *pcd_bi_reg; /* PCDig: busmaster and interrupt regs */
  struct pcd_ca_reg_t *pcd_ca_reg; /* PCDig: camera und acquisition regs */
  pcd_octdata_t *pcd_oct_table; /* PCDig: source scatter gather */
  pcd_dmadata_t *pcd_dma_table; /* PCDig: destination scatter gather */
  u_long *pcd_lut0_table;	/* PCDig: lookup table 0 */
  u_long *pcd_lut1_table;	/* PCDig: lookup table 1 */
  struct pcd_fp_reg_t *pcd_fp_reg; /* PCDig: FPGA programming */
  
  int module;		/* type of installed acq. module */
  int cameras;		/* module supports # of cameras */
  int cur_cam;		/* current camera */
  int fpga_ok;		/* flag: PCDig FPGA's successfully loaded */

  void *ob_base;	/* start of board memory kernel remapped */ 
  void *km_base;	/* start of kernel grab buffer */
  size_t ob_size;	/* size of on-board memory */
  size_t km_size;	/* size of kernel grab buffer */

  short	width;		/* horizontal size of buffer */
  short	height;		/* vertical size of buffer */
  size_t raw_size;	/* size of buffers in bytes */
  size_t paged_size;	/* size of buffer in bytes (aligned) */

  u_long (*get_intadr) (struct _itifg_t*);
  int (*set_intadr) (struct _itifg_t*, u_long);
  u_char (*get_acqmd) (struct _itifg_t*);
  int (*set_acqmd) (struct _itifg_t*, u_char);
  u_long (*get_aqadr) (struct _itifg_t*);
  u_char (*get_pixsz) (struct _itifg_t*);
  u_char (*get_hdec) (struct _itifg_t*);
  int (*set_hdec) (struct _itifg_t*, u_char);
  u_char (*get_vdec) (struct _itifg_t*);
  int (*set_vdec) (struct _itifg_t*, u_char);
  int (*set_fldsel) (struct _itifg_t*, u_char);
  u_char (*get_smstat) (struct _itifg_t*);
  int (*set_vbien) (struct _itifg_t*, u_char);
  int (*set_adrien) (struct _itifg_t*, u_char);
  int (*set_outstrb) (struct _itifg_t*, u_char);
  int (*set_inregenb) (struct _itifg_t*, u_char);
  int (*set_instrbpol) (struct _itifg_t*, u_char);
  int (*set_inregclr) (struct _itifg_t*, u_char);
  int (*set_inpintpol) (struct _itifg_t*, u_char);
  int (*set_inpien) (struct _itifg_t*, u_char);
  u_char (*get_inport) (struct _itifg_t*);
  int (*set_outport) (struct _itifg_t*, u_char);

  /* camera section */
  int cam_timeout;	/* flag: acquisition timeout used in grab/snap mode */
  time_t cam_expire;	/* camera timer expiration interval ( HZ value ) */
  int cam_read_lock;	/* flag: major frame grabber open lock */
  int opmode;		/* operation mode (top / bottom synchronisation) */
  int cam_actual;	/* number of frames already acquired */
  int cam_desired;	/* number of frames to acquire */

  /* io-port section */
  int iop_timeout;	/* flag: io-port timeout used in read mode */
  time_t iop_expire;	/* io-port timer expiration interval ( HZ value ) */
  int iop_read_lock;	/* flag: device is already opened in input mode */
  int write_lock;	/* flag: device is already opened in output mode */
  caddr_t buffer;	/* ptr to kernel space data buffer */
  iti_ssize_t iop_actual;	/* number of bytes already transfered */
  size_t iop_desired;	/* number of bytes to transfer */

  /* acq. block section */
  int blk_actual;	/* current address block to transfer/transferred */
  int blk_desired;	/* how many blocks to transfer */
  int offset;		/* first block is to transfer at address block # */
  int enabled;		/* for this block the transfer is already done */
  int started;		/* if we start in a running acq., we sync with this */

  int iov_actual;	/* number of blocks/pages/lines already transfered */
  int iov_desired;	/* number of blocks/pages/lines to transfered */
  int frst, full, last; /* vectors to transfer in the fist/middle/last block */
  off_t bma1stfld;	/* IC-PCI first interlaced field at */
  off_t bma2ndfld;	/* IC-PCI second interlaced field at */

  void *aio_buffer;
  size_t aio_count;
  iti_ssize_t aio_return;

  /* syscall persistent section */
  /* time marker for the sync rate detection */
  struct timeval lasttime, thistime;
  /* frame times for the snapped images */
  struct timeval timestamp[ITI_SNAPS_MAX];
  /* marker for select(2) in grab mode */
  u_long frames_selected;
  /* fpga download buffer pointer/offset for write(2) */
  caddr_t exo_buffer;
  size_t exo_count;
  int swapped;
};

typedef struct _itifg_t itifg_t;

extern itifg_t *itifg[ITI_BOARDS_MAX]; 

extern int boards; /* number of attached boards */

/* exstime digital camera trigger (first PCDig only) */
extern int need_update;	   /* flag: new exstime value available */
extern u_char new_exsclk;  /* new prescaler value */
extern u_long new_exstime; /* new counter value */
extern u_long new_midpnt;  /* new midpoint value */

extern itifg_t *iti[]; /* export main driver structure */

  /* ioctl copyin parameters */
union iti_inparam_t
{
  /* generic */
  int set_camera;
  union iti_dev_t set_iti_camcnf;
  struct timeval set_timeout;
  /* ic-pci */
  int set_hdec;
  int set_vdec;
  /* am-vs */
  struct vs_lut_t set_vs_lut;
  int set_vs_nref;
  int set_vs_pref;
  /* am-dig */
  int set_dg_lutpg;
  int set_dg_olutpg;
  union dg_lut16_t set_dg_lut16;
  struct dg_lut8_t set_dg_lut8;
  struct timespec set_dg_extime;
  int set_dg_cammd;
  /* pcvision */
  struct pv_lut_t set_pv_lut;
  int set_pv_nref;
  int set_pv_pref;
  /* pcdig */
  int set_pcd_lut0pg;
  int set_pcd_lut1pg;
  union pcd_lut_t set_pcd_lut0;
  union pcd_lut_t set_pcd_lut1;
  int set_pcd_exfree;
  struct timespec set_pcd_extime;
  int set_pcd_cammd;
  int set_pcd_tmode;
  int set_pcd_fcnt;
};

  /* ioctl copyout parameters */
union iti_outparam_t
{
  /* generic */
  short get_width;
  short get_height;
  int get_depth;
  size_t get_rawsize;
  size_t get_pagedsize;
  int get_camera;
  union iti_dev_t get_iti_camcnf;
  struct timeval get_syncrate;
  struct iti_acc_t get_stats;
  /* ic-pci */
  /* am-vs */
  struct vs_lut_t get_vs_lut;
  int get_vs_nref;
  int get_vs_pref;
  /* am-dig */
  int get_dg_lutpg;
  int get_dg_olutpg;
  union dg_lut16_t get_dg_lut16;
  struct dg_lut8_t get_dg_lut8;
  struct timespec get_dg_extime;
  int get_dg_cammd;
  /* pcvision */
  struct pv_lut_t get_pv_lut;
  int get_pv_nref;
  int get_pv_pref;
  /* pcddig */
  int get_pcd_lut0pg;
  int get_pcd_lut1pg;
  union pcd_lut_t get_pcd_lut0;
  union pcd_lut_t get_pcd_lut1;
  struct timespec get_pcd_extime;
  int get_pcd_cammd;
  int get_pcd_tmode;
  int get_pcd_fcnt;
};

/*
 * Prototypes:
 */

/* itifgMain.c */
void
iti_interrupt (itifg_t*);
void
iti_cam_timeout (itifg_t*);
void
iti_task_queue (itifg_t*);

int
iti_open (iti_file_t*, iti_dev_t);
int
iti_close (iti_file_t*, iti_inode_t*, iti_dev_t);
iti_ssize_t
iti_read (iti_file_t*, iti_dev_t, char*, size_t);
iti_ssize_t
iti_write (iti_file_t*, iti_dev_t, const char*, size_t);
iti_loff_t
iti_lseek (iti_file_t*, iti_dev_t, iti_loff_t, int);
int
iti_mmap (iti_file_t*, iti_dev_t, caddr_t, size_t);
int
iti_unmap (iti_file_t*, iti_dev_t, caddr_t, size_t);
int
iti_select (iti_file_t*, iti_dev_t, int);
int
iti_fasync (iti_file_t*, iti_inode_t*, iti_dev_t, int, int);

/* itifgIoctl.c */
int
iti_ioctl (iti_file_t*, iti_dev_t, u_int,
	   union iti_inparam_t*, union iti_outparam_t*);

/* itifgIOP.c */
void
iti_iop_interrupt (itifg_t*);
void
iti_iop_timeout (itifg_t*);
int
iti_iop_open (itifg_t*, iti_file_t*);
int
iti_iop_close (itifg_t*, iti_file_t*);
iti_ssize_t
iti_iop_read (itifg_t*, iti_file_t*, char*, size_t);
int
iti_iop_write (itifg_t*, iti_file_t*, const char*, size_t);

/* itifgAux.c */
enum _iti_error_t
{
  EITI_ERR_NO,
  EITI_MASK,
  EITI_RANGE,
  EITI_NOSUPPORT,
  EITI_POINTER,
  EITI_BUSY
};

void
iti_error(itifg_t*, const char*);

int
iti_decode_devno (iti_dev_t, int*, int*);

typedef enum
{
  ITI_EIRQ_MAIN_OPEN,	/* at FIRST open (enable main irq's) */ 
  ITI_EIRQ_DATA_OPEN,	/* to get every 64k (image-) pieces an interrupt */
  ITI_EIRQ_PORT_OPEN	/* to check for data (enable input port irq) */
} iti_eirq_t;

int
iti_enable_irq (itifg_t*, iti_eirq_t);

typedef enum
{
  ITI_DIRQ_MAIN_CLOSE,	/* at LAST close (disable all irq's) */
  ITI_DIRQ_DATA_CLOSE,	/* all camera dependent irq's (vb, eoacq, trig) */
  ITI_DIRQ_PORT_CLOSE,	/* disable only the input port irq */
} iti_dirq_t;

int
iti_disable_irq (itifg_t*, iti_dirq_t);

int
iti_cr_init (itifg_t*);

enum _iti_rmap_t
{
  ICR_REGS,
  CR_REGS,
  NUM_REGS
};  

int
iti_pci_init (itifg_t*, void*);
int
iti_map_init (itifg_t*, void*);
int
iti_cap_init (itifg_t*);
int
iti_dev_init (itifg_t*);

int
iti_alloc_board (itifg_t*);
void
iti_free_board (itifg_t*);
int
iti_attach (int, void*);
int
iti_detach (void);

/* itifgBuf.c */
int
iti_bma_restart (itifg_t*);
int
iti_bma_start (itifg_t*);
int
iti_bma_stop (itifg_t*);

typedef enum 
{
  ITI_CLEAR_BLACK,	/* fill frame with black pixels */
  ITI_CLEAR_GREY	/* fill frame with grey pixels */
} iti_clear_t;   

void
iti_clear (itifg_t*, caddr_t, size_t, iti_clear_t, int);
int
iti_clear_sglist (itifg_t*);

int
iti_src_init (itifg_t*);
int
iti_dst_init (itifg_t*, caddr_t, size_t, int);
int
iti_fmt_init (itifg_t*);
int
iti_buf_init (itifg_t*);

/* itifgAM.c */
int
iti_get_lut16 (itifg_t*, char, union iti_lut16_t*);
int
iti_set_lut16 (itifg_t*, char, union iti_lut16_t*);

typedef enum 
{
  ITI_MODE_LINEAR,
  ITI_MODE_NORM_TO8,
  ITI_MODE_NORM_TO16
} iti_lut16md_t;

int
iti_lin_lut16 (itifg_t*, char, iti_lut16md_t);

u_char
icp_get_modid (itifg_t*);
int
icp_amvs_startup (itifg_t*);
int
icp_amvs_init (itifg_t*);
int
icp_amdg_startup (itifg_t*);
int
icp_amdg_init (itifg_t*);
int
icp_ampv_startup (itifg_t*);
int
icp_ampv_init (itifg_t*);
int
pcd_viam_startup (itifg_t*);
int
pcd_viam_init (itifg_t*);

typedef enum
{
  ITI_UNPAGED,		/* only small amount of memory */
  ITI_PAGED		/* more space, but can block */
} iti_alloc_t;

void*
iti_malloc (size_t, iti_alloc_t);
void
iti_free (void*, iti_alloc_t);

typedef enum
{
  ITI_CAMERA,
  ITI_IOPORT
} iti_sleep_t;

/* osxxxxxXX.c */
int
iti_add_timer (itifg_t*, time_t, iti_sleep_t);
int
iti_del_timer (itifg_t*, iti_sleep_t);
int
iti_sleep (itifg_t*, time_t, iti_sleep_t);
int
iti_block_wakeup (itifg_t*, iti_sleep_t);
int
iti_select_wakeup (itifg_t*, iti_sleep_t);
int
iti_marktq (itifg_t*);
void
iti_lock (itifg_t*);
void
iti_unlock (itifg_t*);
void
iti_irq_lock (itifg_t*);
void
iti_irq_unlock (itifg_t*);
int
iti_get_time (struct timeval*);
int
iti_get_major (iti_dev_t, int*);
int
iti_get_minor (iti_dev_t, int*);
int
iti_virt_to_phys (caddr_t, u_long*, u_long*, int);
int
iti_area_locked (caddr_t);
int
iti_lock_page (u_long, int);
int
iti_unlock_page (u_long, int);
int
iti_remap_page (caddr_t, u_long);
int
iti_init_fasync (itifg_t*, iti_file_t*, iti_inode_t*, int, int);
int
iti_kill_fasync (itifg_t*, iti_sleep_t);

int
iti_os_init (itifg_t*);
int
iti_os_pci_init (itifg_t*, void*);
int
iti_os_map_init (itifg_t*, void*, u_long*, long*);

#endif /* linux && __KERNEL__ || !OS_DEFINED */

#endif /* _ITIFGINTD_H */

